function normalize_and_reduce()
  local sample = renoise.song().selected_sample
  if not sample then
    renoise.app():show_error("No sample selected!")
    return
  end

  local buffer = sample.sample_buffer
  if not buffer.has_sample_data then
    renoise.app():show_error("The selected sample has no data!")
    return
  end

  buffer:prepare_sample_data_changes()

  -- Find the maximum amplitude
  local max_amplitude = 0
  for channel = 1, buffer.number_of_channels do
    for frame = 1, buffer.number_of_frames do
      local sample_value = math.abs(buffer:sample_data(channel, frame))
      if sample_value > max_amplitude then
        max_amplitude = sample_value
      end
    end
  end

  -- Normalize and reduce volume
  if max_amplitude > 0 then
    local normalization_factor = 1 / max_amplitude
    for channel = 1, buffer.number_of_channels do
      for frame = 1, buffer.number_of_frames do
        local sample_value = buffer:sample_data(channel, frame)
        buffer:set_sample_data(channel, frame, sample_value * normalization_factor * 0.25) -- -12 dB
      end
    end
  end

  buffer:finalize_sample_data_changes()
end

-- Add to menu
renoise.tool():add_menu_entry {
  name = "Sample Editor:Tools:Normalize to -12 dB",
  invoke = normalize_and_reduce
}

-- Add to keybindings
renoise.tool():add_keybinding {
  name = "Sample Editor:Tools:Normalize to -12 dB",
  invoke = normalize_and_reduce
}